/************************************************************************/
/*                                                                      */
/*    TMP2 Quick Start Demo  using DTWI                                 */
/*                                                                      */
/*                                                                      */
/************************************************************************/
/*    Author:    Jesse Njuguna                                          */
/*    Copyright 2015, Digilent Inc.                                     */
/************************************************************************/
/* 
*
* Copyright (c) 2013-2014, Digilent <www.digilentinc.com>
* Contact Digilent for the latest version.
*
* This program is free software; distributed under the terms of 
* BSD 3-clause license ("Revised BSD License", "New BSD License", or "Modified BSD License")
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* 1.    Redistributions of source code must retain the above copyright notice, this
*        list of conditions and the following disclaimer.
* 2.    Redistributions in binary form must reproduce the above copyright notice,
*        this list of conditions and the following disclaimer in the documentation
*        and/or other materials provided with the distribution.
* 3.    Neither the name(s) of the above-listed copyright holder(s) nor the names
*        of its contributors may be used to endorse or promote products derived
*        from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*/
/************************************************************************/
/*  Revision History:                                                   */
/*   03/16/2015 (JesseN): Created                                       */
/************************************************************************/

#include <DTWI.h>

#define TMP2_ADDRESS     0x4B
#define FLOATING_PT_VAL  0.0625



typedef enum{
  NONE,
  START,
  BEGINMASTER,
  STARTMASTERREAD,
  STOPMASTER,
  ERROR,
}TWISTATE;

TWISTATE TMP2State = START;

DTWI0 dtwi0;

byte dataArr[2] = {0};

/***    void setup()
 *
 *    Parameters:
 *          None
 *              
 *    Return Values:
 *          None
 *
 *    Description: 
 *    
 *      Arduino setup routine
 * ------------------------------------------------------------ */
void setup()
{
  Serial.begin(9600);
}

/***    void loop()
 *
 *    Parameters:
 *          None
 *              
 *    Return Values:
 *          None
 *
 *    Description: 
 *    
 *      Arduino loop routine
 * ------------------------------------------------------------ */
void loop()
{
  
  switch(TMP2State)
  {
   case NONE:
     break;
   case START:
        Serial.println("PmodTMP2 Demo Using DTWI Library");
        Serial.println("Author: Jesse Njuguna");
        TMP2State = BEGINMASTER;
     break;
   case BEGINMASTER:
          if(dtwi0.beginMaster())
          {
            Serial.println("DTWI0 Baud Rate Configured and I2C Controller turned On");
            TMP2State = STARTMASTERREAD;
          }
     break;
   case STARTMASTERREAD:
         if(dtwi0.startMasterRead(TMP2_ADDRESS, sizeof(dataArr)))
         {
           Serial.println("Start Temp Reading");
           uint32_t bytes_read = dtwi0.read(dataArr , sizeof(dataArr));
           temp_convert();
           TMP2State = STOPMASTER;
         }
     break;
   case STOPMASTER:
         if(dtwi0.stopMaster())
         {
            dtwi0.endMaster();
            //clean the read buffer
            dtwi0.abort();
            TMP2State = NONE;
         }
     break;
   default: 
     break;
  }  
}
\
/***    void temp_convert()
 *
 *    Parameters:
 *          None
 *              
 *    Return Values:
 *          None
 *
 *    Description: 
 *    
 *     Converts temp data from TMP2 to degrees Celsius and Farenheit
 * ------------------------------------------------------------------ */
void temp_convert()
{
  double TMP2_tempF, TMP2_tempC;
  uint16_t tempTwosData = dataArr[0];
  tempTwosData = tempTwosData << 8;
  tempTwosData = tempTwosData | dataArr[1];
  
  tempTwosData = (tempTwosData >> 3) & 0xFFFF;
  TMP2_tempC = tempTwosData * FLOATING_PT_VAL;
  
  TMP2_tempF = (TMP2_tempC * (1.8)) + 32;
  
  Serial.print("Temperature in C: ");
  Serial.println(TMP2_tempC);
  Serial.print("Temperature in F: ");
  Serial.println(TMP2_tempF);
   
}
 
























